@@ -1,7 +1,7 @@
-/* $Id: actions.c,v 1.22 2009/06/04 17:01:06 imil Exp $ */
+/* $Id: actions.c,v 1.31 2010/01/23 14:30:07 imil Exp $ */
 
 /*
- * Copyright (c) 2009 The NetBSD Foundation, Inc.
+ * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * This code is derived from software contributed to The NetBSD Foundation
@@ -53,21 +53,24 @@ static int			upgrade_type = UPGRADE_NONE;
 int
 check_yesno(void)
 {
-	int c, r = 'n';
-	
+	int c, r = 0;
+
 	if (yesflag)
 		return 1;
-	
+	else if (noflag)
+	    	return 0;
+
 	printf(MSG_PROCEED);
 
-	while((c = getchar()) != '\n' && c != EOF) /* avoid residual char */
-		if (c == 'y')
-			r = 'y';
-	
-	if (r == 'y')
-		return 1;
-	else
-		return 0;
+	if ((c = getchar()) == 'y')
+	    	r = 1;
+	else if (c == '\n' || c == EOF)
+	    	return 0;
+
+	/* avoid residual char */
+	while((c = getchar()) != '\n' && c != EOF)
+		continue;
+	return r;
 }
 
 /* package removal */
@@ -77,13 +80,17 @@ do_pkg_remove(Deptreehead *removehead)
 	Pkgdeptree *premove;
 
 	SLIST_FOREACH(premove, removehead, next) {
+		/* file not available in the repository */
+	    	if (premove->file_size == -1)
+		    	continue;
+
 		if (premove->depname == NULL)
 			/* SLIST corruption, badly installed package */
 			continue;
 		printf(MSG_REMOVING, premove->depname);
 #ifdef DEBUG
 		printf("%s -f %s\n", PKG_DELETE, premove->depname);
-#else		
+#else
 		fexec(PKG_DELETE, "-f", premove->depname, NULL);
 #endif
 	}
@@ -96,40 +103,46 @@ pkg_download(Deptreehead *installhead)
 	Pkgdeptree	*pinstall;
 	struct stat	st;
 	Dlfile		*dlpkg;
-	char		*pkgurl, query[BUFSIZ], pkgpath[BUFSIZ];
+	char		pkg[BUFSIZ], query[BUFSIZ];
 
 	printf(MSG_DOWNLOAD_PKGS);
 
 	SLIST_FOREACH(pinstall, installhead, next) {
-		snprintf(pkgpath, BUFSIZ,
-			"%s/%s%s", pkgin_cache, pinstall->depname, PKG_EXT);
+		snprintf(pkg, BUFSIZ,
+		    "%s/%s%s", pkgin_cache, pinstall->depname, PKG_EXT);
 
-		/* package not downloaded or wrong size */
-		if (stat(pkgpath, &st) < 0 || st.st_size != pinstall->file_size) {
+		/* already fully downloaded */
+		if (stat(pkg, &st) == 0 && st.st_size == pinstall->file_size)
+		    	continue;
 
-			XMALLOC(pkgurl, BUFSIZ * sizeof(char));
+		umask(DEF_UMASK);
+		if ((fp = fopen(pkg, "w")) == NULL)
+			err(EXIT_FAILURE, MSG_ERR_OPEN, pkg);
 
-			snprintf(query, BUFSIZ, PKG_URL, pinstall->depname);
-			/* retreive repository for package  */
-			if (pkgindb_doquery(query, pdb_get_value, pkgurl) != 0)
-				errx(EXIT_FAILURE, MSG_PKG_NO_REPO, 
-					pinstall->depname);
+		snprintf(query, BUFSIZ, PKG_URL, pinstall->depname);
+		/* retrieve repository for package  */
+		if (pkgindb_doquery(query, pdb_get_value, pkg) != 0)
+			errx(EXIT_FAILURE, MSG_PKG_NO_REPO, pinstall->depname);
 
-			XSNPRINTF(pkgurl, BUFSIZ,
-				"%s/%s%s", pkgurl, pinstall->depname, PKG_EXT);
-			dlpkg = download_file(pkgurl, NULL);
+		strlcat(pkg, "/", sizeof(pkg));
+		strlcat(pkg, pinstall->depname, sizeof(pkg));
+		strlcat(pkg, PKG_EXT, sizeof(pkg));
 
-			XFREE(pkgurl);
-
-			umask(DEF_UMASK);
-			
-			if ((fp = fopen(pkgpath, "w")) == NULL)
-				err(EXIT_FAILURE, MSG_ERR_OPEN, pkgpath);
-			
-			fwrite(dlpkg->buf, dlpkg->size, 1, fp);
-			
+		if ((dlpkg = download_file(pkg, NULL)) == NULL) {
+			fprintf(stderr, MSG_PKG_NOT_AVAIL, pinstall->depname);
+			if (!check_yesno())
+				errx(EXIT_FAILURE, MSG_PKG_NOT_AVAIL,
+				    pinstall->depname);
+			pinstall->file_size = -1;
 			fclose(fp);
+			continue;
 		}
+
+		fwrite(dlpkg->buf, dlpkg->size, 1, fp);
+		fclose(fp);
+
+		XFREE(dlpkg->buf);
+		XFREE(dlpkg);
 	} /* download loop */
 }
 
@@ -145,9 +158,13 @@ do_pkg_install(Deptreehead *installhead)
 	char		pkgpath[BUFSIZ];
 
 	printf(MSG_INSTALL_PKG);
-	
+
 	SLIST_FOREACH(pinstall, installhead, next) {
 
+		/* file not available in the repository */
+	    	if (pinstall->file_size == -1)
+		    	continue;
+
 		printf(MSG_INSTALLING, pinstall->depname);
 		snprintf(pkgpath, BUFSIZ,
 			"%s/%s%s", pkgin_cache, pinstall->depname, PKG_EXT);
@@ -158,7 +175,7 @@ do_pkg_install(Deptreehead *installhead)
 		fexec(PKG_ADD, "-f", pkgpath, NULL);
 #endif
 	} /* installation loop */
-}	
+}
 
 /* build the output line */
 char *
@@ -174,10 +191,9 @@ action_list(char *flatlist, char *str)
 			return flatlist;
 
 		newsize = strlen(str) + strlen(flatlist) + 2;
-		XREALLOC(newlist, newsize * sizeof(char));
-		snprintf(newlist, newsize, "%s %s", flatlist, str);
-
-		XFREE(flatlist);
+		newlist = realloc(flatlist, newsize * sizeof(char));
+		strlcat(newlist, " ", newsize);
+		strlcat(newlist, str, newsize);
 	}
 
 	return newlist;
@@ -188,9 +204,14 @@ static int
 pkg_met_reqs(Impacthead *impacthead)
 {
 	int			met_reqs = 1, foundreq;
-	Pkgimpact	*pimpact, *impactprov;
-	Plisthead	*requireshead, *provideshead;
-	Pkglist		*requires, *provides;
+	Pkgimpact	*pimpact;
+	Plisthead	*requireshead;
+	Pkglist		*requires;
+#ifdef CHECK_PROVIDES
+	Pkgimpact	*impactprov;
+	Plisthead	*provideshead;
+	Pkglist		*provides;
+#endif
 	struct stat	sb;
 	char		query[BUFSIZ];
 
@@ -209,11 +230,12 @@ pkg_met_reqs(Impacthead *impacthead)
 			foundreq = 0;
 
 			/* for performance sake, first check basesys */
-			if ((strstr(requires->pkgname, LOCALBASE)) == NULL) {
+			if ((strncmp(requires->pkgname, LOCALBASE,
+				    sizeof(LOCALBASE) - 1)) != 0) {
 				if (stat(requires->pkgname, &sb) < 0) {
 					printf(MSG_REQT_NOT_PRESENT,
-						requires->pkgname, pimpact->pkgname); 
-				
+						requires->pkgname, pimpact->pkgname);
+
 					met_reqs = 0;
 				}
 				/* was a basysfile, no need to check PROVIDES */
@@ -224,8 +246,9 @@ pkg_met_reqs(Impacthead *impacthead)
 			 * PROVIDES in pkg_summary(5). This is a known issue and will
 			 * hopefuly be fixed.
 			 */
+#ifndef CHECK_PROVIDES
 			continue;
-
+#else
 			/* search what local packages provide */
 			provideshead = rec_pkglist(LOCAL_PROVIDES);
 			SLIST_FOREACH(provides, provideshead, next) {
@@ -234,13 +257,12 @@ pkg_met_reqs(Impacthead *impacthead)
 						strlen(requires->pkgname)) == 0) {
 
 					foundreq = 1;
-					
+
 					/* found, no need to go further*/
 					break;
 				} /* match */
 			} /* SLIST_FOREACH LOCAL_PROVIDES */
 			free_pkglist(provideshead);
-			XFREE(provideshead);
 
 			/* REQUIRES was not found on local packages, try impact list */
 			if (!foundreq) {
@@ -252,22 +274,21 @@ pkg_met_reqs(Impacthead *impacthead)
 
 					if (provideshead == NULL)
 						continue;
-				
+
 					/* then parse provides list for every package */
 					SLIST_FOREACH(provides, provideshead, next) {
 						if (strncmp(provides->pkgname,
 								requires->pkgname,
 								strlen(requires->pkgname)) == 0) {
-							
+
 							foundreq = 1;
-							
+
 							/* found, no need to go further
 							   return to impactprov list */
 							break;
 						} /* match */
 					}
 					free_pkglist(provideshead);
-					XFREE(provideshead);
 
 					if (foundreq) /* exit impactprov list loop */
 						break;
@@ -288,11 +309,9 @@ pkg_met_reqs(Impacthead *impacthead)
 
 				foundreq = 1;
 			}
-
+#endif
 		} /* SLIST_FOREACH requires */
-		
 		free_pkglist(requireshead);
-		XFREE(requireshead);
 	} /* 1st impact SLIST_FOREACH */
 
 	return met_reqs;
@@ -308,28 +327,28 @@ pkg_has_conflicts(Plisthead *conflictshead, Pkgimpact *pimpact)
 
 	if (conflictshead == NULL)
 		return 0;
-		
+
 	/* check conflicts */
 	SLIST_FOREACH(conflicts, conflictshead, next) {
 		if (pkg_match(conflicts->pkgname, pimpact->pkgname)) {
-			
+
 			/* got a conflict, retrieve conflicting local package */
 			snprintf(query, BUFSIZ,
 				GET_CONFLICT_QUERY, conflicts->pkgname);
-			
+
 			XMALLOC(conflict_pkg, BUFSIZ * sizeof(char));
 			if (pkgindb_doquery(query,
 					pdb_get_value, conflict_pkg) == 0)
-				
+
 				printf(MSG_CONFLICT_PKG,
 					pimpact->pkgname, conflict_pkg);
-			
+
 			XFREE(conflict_pkg);
 
 			has_conflicts = 1;
 		} /* match conflict */
 	} /* SLIST_FOREACH conflicts */
-	
+
 	return has_conflicts;
 }
 
@@ -364,7 +383,7 @@ pkgin_install(char **pkgargs)
 
 	/* conflicts list */
 	conflictshead = rec_pkglist(LOCAL_CONFLICTS);
-	
+
 	/* browse impact tree */
 	SLIST_FOREACH(pimpact, impacthead, next) {
 
@@ -385,7 +404,7 @@ pkgin_install(char **pkgargs)
 		if (stat(pkgpath, &st) < 0 || st.st_size != pimpact->file_size)
 			file_size += pimpact->file_size;
 		size_pkg += pimpact->size_pkg;
-		
+
 		switch (pimpact->action) {
 		case TOUPGRADE:
 			upgradenum++;
@@ -403,7 +422,6 @@ pkgin_install(char **pkgargs)
 	}
 	/* free conflicts list */
 	free_pkglist(conflictshead);
-	XFREE(conflictshead);
 
 	(void)humanize_number(h_fsize, H_BUF, (int64_t)file_size, "",
 		HN_AUTOSCALE, HN_B | HN_NOSPACE | HN_DECIMAL);
@@ -436,7 +454,7 @@ pkgin_install(char **pkgargs)
 
 	} else
 		printf(MSG_NOTHING_TO_UPGRADE);
-	
+
 	if (installnum > 0) {
 		/* record ordered install list */
 		installhead = order_install(impacthead);
@@ -501,15 +519,13 @@ pkgin_remove(char **pkgargs)
 
 		if ((pkgname = find_exact_pkg(plisthead, *ppkgargs)) == NULL) {
 			printf(MSG_PKG_NOT_INSTALLED, *ppkgargs);
-			XFREE(pkgname);
 			continue;
 		}
 		XSTRDUP(ppkg, pkgname);
 		trunc_str(ppkg, '-', STR_BACKWARD);
 
 		/* record full reverse dependency list for package */
-		full_dep_tree(ppkg, LOCAL_REVERSE_DEPS,
-			&pdphead, __func__, 0);
+		full_dep_tree(ppkg, LOCAL_REVERSE_DEPS, &pdphead);
 
 		XFREE(ppkg);
 
@@ -530,9 +546,7 @@ pkgin_remove(char **pkgargs)
 
 		/* add package itself */
 		XMALLOC(pdp, sizeof(Pkgdeptree));
-		XSTRDUP(pdp->depname, pkgname);
-
-		XFREE(pkgname);
+		pdp->depname = pkgname;
 
 		if (SLIST_EMPTY(&pdphead))
 			/* identify unique package, don't cut it when ordering */
@@ -547,7 +561,6 @@ pkgin_remove(char **pkgargs)
 	} /* for pkgargs */
 
 	free_pkglist(plisthead);
-	XFREE(plisthead);
 
 	/* order remove list */
 	removehead = order_remove(&pdphead);
@@ -556,18 +569,18 @@ pkgin_remove(char **pkgargs)
 		deletenum++;
 		todelete = action_list(todelete, pdp->depname);
 	}
-	
+
 	if (todelete != NULL) {
 		printf(MSG_PKGS_TO_DELETE, deletenum, todelete);
 		if (check_yesno()) {
 			do_pkg_remove(removehead);
-			
+
 			update_db(LOCAL_SUMMARY, NULL);
 
 			rc = EXIT_SUCCESS;
 		} else
 			rc = EXIT_FAILURE;
-		
+
 	} else {
 		printf(MSG_NO_PKGS_TO_DELETE);
 		rc = EXIT_SUCCESS;
@@ -598,7 +611,7 @@ narrow_match(Plisthead *remoteplisthead,
 
 		if (strlen(rpkgname) == pkglen &&
 			strncmp(pkgname, rpkgname, pkglen) == 0) {
-			
+
 			for (i = 0;
 				 i < fullpkglen && fullpkgname[i] == pkglist->pkgname[i];
 				i++);
@@ -607,7 +620,7 @@ narrow_match(Plisthead *remoteplisthead,
 				matchlen = i;
 				XSTRDUP(best_match, pkglist->pkgname);
 			}
-			
+
 		}
 
 		XFREE(rpkgname);
@@ -634,9 +647,9 @@ record_upgrades(Plisthead *plisthead, Plisthead *remoteplisthead)
 		XSTRDUP(pkgargs[count], pkglist->pkgname);
 		trunc_str(pkgargs[count], '-', STR_BACKWARD);
 
-		pkgargs[count] = narrow_match(remoteplisthead, 
+		pkgargs[count] = narrow_match(remoteplisthead,
 			pkgargs[count], pkglist->pkgname);
-		
+
 		if (pkgargs[count] == NULL)
 			continue;
 
@@ -681,8 +694,6 @@ pkgin_upgrade(int uptype)
 		if (uptype == UPGRADE_ALL) {
 
 			free_pkglist(localplisthead);
-			XFREE(localplisthead);
-
 			free_list(pkgargs);
 			/* record keep list */
 			pkgargs = record_upgrades(keeplisthead, remoteplisthead);
@@ -694,8 +705,5 @@ pkgin_upgrade(int uptype)
 	free_list(pkgargs);
 
 	free_pkglist(remoteplisthead);
-	XFREE(remoteplisthead);
-
 	free_pkglist(keeplisthead);
-	XFREE(keeplisthead);
 }
