@@ -1,7 +1,7 @@
-/* $Id: messages.h,v 1.8 2009/06/05 17:37:39 imil Exp $ */
+/* $Id: messages.h,v 1.15 2010/01/03 14:07:29 imil Exp $ */
 
 /*
- * Copyright (c) 2009 The NetBSD Foundation, Inc.
+ * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * This code is derived from software contributed to The NetBSD Foundation
@@ -41,7 +41,7 @@
 #define MSG_PKG_ARGS_UNKEEP "specify at least one package to unkeep"
 #define MSG_MISSING_SRCH "missing search string"
 
-#define MSG_USAGE "Usage: %s [-fhvy] command [package ...]\n"
+#define MSG_USAGE "Usage: %s [-fhlnvy] command [package ...]\n"
 #define MSG_CMDS_SHORTCUTS "\nCommands and shortcuts:\n"
 
 
@@ -85,7 +85,8 @@
 #define MSG_DIRECT_DEPS_FOR "direct dependencies for %s\n"
 
 /* download.c */
-#define MSG_DOWNLOADING "downloading %s: %.f%%\r"
+#define MSG_DOWNLOADING "downloading %s:   0%%"
+#define MSG_DOWNLOADING_PCT "\b\b\b\b%3d%%"
 
 /* autoremove.c */
 #define MSG_AUTOREMOVE_WARNING "\
@@ -94,15 +95,17 @@ in order to remove packages from the autoremove list, flag those with the -k mod
 #define MSG_ALL_KEEP_PKGS "all packages are marked as \"keepable\"."
 #define MSG_AUTOREMOVE_PKGS "%d packages to be autoremoved: %s\n"
 #define MSG_MARK_PKG_KEEP "%s is marked as non-autoremovable\n"
-#define MSG_MARKING_PKG_KEEP "marking %s as non auto-removeable\n"
-#define MSG_UNMARKING_PKG_KEEP "marking %s as auto-removeable\n"
+#define MSG_MARKING_PKG_KEEP "marking %s as non auto-removable\n"
+#define MSG_UNMARKING_PKG_KEEP "marking %s as auto-removable\n"
 
 /* summary.c */
-#define MSG_UPDATING_DB "updating database: %.f%%\r"
+#define MSG_UPDATING_DB "updating database:   0%%"
+#define MSG_UPDATING_DB_PCT "\b\b\b\b%3d%%"
 #define MSG_CLEANING_DB_FROM_REPO "cleaning database from %s entries...\n"
 #define MSG_PROCESSING_LOCAL_SUMMARY "processing local summary...\n"
 #define MSG_DB_IS_UP_TO_DATE "database for %s is up-to-date\n"
 #define MSG_PROCESSING_REMOTE_SUMMARY "processing remote summary (%s)...\n"
+#define MSG_COULDNT_FETCH "Could not fetch %s\n"
 
 /* impact.c */
 #define MSG_GREATER_VERSION \
@@ -115,3 +118,7 @@ please re-run %s with a package name matching one of the following:\n"
 
 /* pkglist.c */
 #define MSG_IS_INSTALLED_CODE "\n=: package is installed and up-to-date\n<: package is installed but newer version is available\n>: installed package has a greater version than available package\n"
+
+/* fsops.c */
+#define MSG_TRANS_FAILED "Failed to translate %s in repository config file"
+#define MSG_INVALID_REPOS "Invalid repository: %s"
