@@ -1,7 +1,7 @@
-/* $Id: pkglist.c,v 1.12 2009/06/06 18:03:56 imil Exp $ */
+/* $Id: pkglist.c,v 1.21 2010/01/23 14:30:07 imil Exp $ */
 
 /*
- * Copyright (c) 2009 The NetBSD Foundation, Inc.
+ * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * This code is derived from software contributed to The NetBSD Foundation
@@ -48,6 +48,7 @@ free_pkglist(Plisthead *plisthead)
 		XFREE(plist->comment);
 		XFREE(plist);
 	}
+	XFREE(plisthead);
 }
 
 /* sqlite callback, record package list */
@@ -56,7 +57,7 @@ pdb_rec_pkglist(void *param, int argc, char **argv, char **colname)
 {
 	Pkglist		*plist;
 	Plisthead 	*plisthead = (Plisthead *)param;
-	
+
 	if (argv == NULL)
 		return PDB_ERR;
 
@@ -95,7 +96,7 @@ pdb_rec_pkglist(void *param, int argc, char **argv, char **colname)
 		plist->comment = NULL;
 
 	SLIST_INSERT_HEAD(plisthead, plist, next);
-		
+
 	return PDB_OK;
 }
 
@@ -105,58 +106,99 @@ rec_pkglist(const char *pkgquery)
 	Plisthead *plisthead;
 
 	XMALLOC(plisthead, sizeof(Plisthead));
-	
+
 	SLIST_INIT(plisthead);
-	
+
 	if (pkgindb_doquery(pkgquery, pdb_rec_pkglist, plisthead) == 0)
 		return plisthead;
-	else
-		return NULL;
+
+	XFREE(plisthead);
+	return NULL;
 }
 
 static int
 pkg_is_installed(Plisthead *plisthead, char *pkgname)
 {
 	Pkglist		*pkglist;
-	char		*pkg1, *pkg2;
-	int			rc;
+	int			cmplen;
+	char		*dashp;
+
+	if ((dashp = strrchr(pkgname, '-')) == NULL)
+		return -1;
+
+	cmplen = dashp - pkgname;
 
 	SLIST_FOREACH(pkglist, plisthead, next) {
-		XSTRDUP(pkg1, pkglist->pkgname);
-		XSTRDUP(pkg2, pkgname);
+	    	dashp = strrchr(pkglist->pkgname, '-');
+		if (dashp == NULL)
+		    	err(EXIT_FAILURE, "oops");
 
-		trunc_str(pkg1, '-', STR_BACKWARD);
-		trunc_str(pkg2, '-', STR_BACKWARD);
+	    	/* make sure foo-1 does not match foo-bin-1 */
+	    	if ((dashp - pkglist->pkgname) != cmplen)
+		    	continue;
 
-		if (strcmp(pkg1, pkg2) == 0) {
+		if (strncmp(pkgname, pkglist->pkgname, cmplen) != 0)
+		    	continue;
 
-			if (strcmp(pkglist->pkgname, pkgname) == 0)
-				return 0;
+		if (strcmp(dashp, pkgname + cmplen) == 0)
+			return 0;
 
-			rc = version_check(pkglist->pkgname, pkgname); 
-
-			if (rc == 1)
-				return 1;
-			if (rc == 2)
-				return 2;
-		}
+		return version_check(pkglist->pkgname, pkgname);
 	}
 
 	return -1;
 }
 
 void
-list_pkgs(const char *pkgquery)
+list_pkgs(const char *pkgquery, int lstype)
 {
 	Pkglist 	*plist;
-	Plisthead 	*plisthead;
+	Plisthead 	*plisthead, *localplisthead = NULL;
+	int			rc;
+	char		pkgstatus, outpkg[BUFSIZ];
+
+	/* list installed packages + status */
+	if (lstype == PKG_LLIST_CMD && lslimit != '\0') {
+		localplisthead = rec_pkglist(LOCAL_PKGS_QUERY);
+
+		if (localplisthead == NULL)
+			return;
+
+		if ((plisthead = rec_pkglist(REMOTE_PKGS_QUERY)) != NULL) {
+
+			SLIST_FOREACH(plist, plisthead, next) {
+				rc = pkg_is_installed(localplisthead, plist->pkgname);
+
+				pkgstatus = '\0';
+
+				if (lslimit == PKG_EQUAL && rc == 0)
+					pkgstatus = PKG_EQUAL;
+				if (lslimit == PKG_GREATER && rc == 1)
+					pkgstatus = PKG_GREATER;
+				if (lslimit == PKG_LESSER && rc == 2)
+					pkgstatus = PKG_LESSER;
+
+				if (pkgstatus != '\0') {
+					snprintf(outpkg, BUFSIZ, "%s %c",
+						plist->pkgname, pkgstatus);
+					printf("%-20s %s\n", outpkg, plist->comment);
+				}
+
+			}
+			free_pkglist(plisthead);
+		}
+		free_pkglist(localplisthead);
+		return;
+	} /* lstype == LLIST && status */
 
 	if ((plisthead = rec_pkglist(pkgquery)) != NULL) {
 		SLIST_FOREACH(plist, plisthead, next)
 			printf("%-20s %s\n", plist->pkgname, plist->comment);
-		
+
 		free_pkglist(plisthead);
-	}		
+	}
+
+
 }
 
 void
@@ -166,7 +208,7 @@ search_pkg(const char *pattern)
 	Plisthead 	*plisthead, *localplisthead;
 	regex_t		re;
 	int			rc;
-	char		eb[64], is_inst[3], outpkg[BUFSIZ];
+	char		eb[64], is_inst, outpkg[BUFSIZ];
 
 	localplisthead = rec_pkglist(LOCAL_PKGS_QUERY);
 
@@ -178,36 +220,33 @@ search_pkg(const char *pattern)
 		}
 
 		SLIST_FOREACH(plist, plisthead, next) {
-			is_inst[0] = '\0';
+			is_inst = '\0';
 
 			if (regexec(&re, plist->pkgname, 0, NULL, 0) == 0 ||
 				regexec(&re, plist->comment, 0, NULL, 0) == 0) {
 
 				if (localplisthead != NULL) {
 					rc = pkg_is_installed(localplisthead, plist->pkgname);
-					
+
 					if (rc == 0)
-						XSTRCPY(is_inst, "=");
+						is_inst = PKG_EQUAL;
 					if (rc == 1)
-						XSTRCPY(is_inst, ">");
+						is_inst = PKG_GREATER;
 					if (rc == 2)
-						XSTRCPY(is_inst, "<");
+						is_inst = PKG_LESSER;
 
 				}
 
-				snprintf(outpkg, BUFSIZ, "%s %s", plist->pkgname, is_inst);
+				snprintf(outpkg, BUFSIZ, "%s %c", plist->pkgname, is_inst);
 
 				printf("%-20s %s\n", outpkg, plist->comment);
 			}
 		}
 
 		free_pkglist(plisthead);
-		XFREE(plisthead);
 
-		if (localplisthead != NULL) {
+		if (localplisthead != NULL)
 			free_pkglist(localplisthead);
-			XFREE(localplisthead);
-		}
 
 		regfree(&re);
 
@@ -222,10 +261,8 @@ find_exact_pkg(Plisthead *plisthead, const char *pkgarg)
 	char	*pkgname, *tmppkg;
 	int		tmplen, exact;
 
-	XSTRDUP(pkgname, pkgarg);
-
 	/* truncate argument if it contains a version */
-	exact = exact_pkgfmt(pkgname);
+	exact = exact_pkgfmt(pkgarg);
 
 	/* check for package existence */
 	SLIST_FOREACH(pkglist, plisthead, next) {
@@ -239,14 +276,12 @@ find_exact_pkg(Plisthead *plisthead, const char *pkgarg)
 			 * and set len to tmppkg
 			 */
 			trunc_str(tmppkg, '-', STR_BACKWARD);
-			tmplen = strlen(tmppkg);
-		} else
-			/* tmplen = strlen("foo-1{.vers}") */
-			tmplen = strlen(pkgname);
+		}
+		/* tmplen = strlen("foo-1{.vers}") */
+		tmplen = strlen(tmppkg);
 
-		if (strlen(pkgname) == tmplen && 
-			strncmp(tmppkg, pkgname, tmplen) == 0) {
-			XFREE(pkgname);
+		if (strlen(pkgarg) == tmplen &&
+			strncmp(tmppkg, pkgarg, tmplen) == 0) {
 			XFREE(tmppkg);
 
 			XSTRDUP(pkgname, pkglist->pkgname);
@@ -255,8 +290,6 @@ find_exact_pkg(Plisthead *plisthead, const char *pkgarg)
 		XFREE(tmppkg);
 	}
 
-	XFREE(pkgname);
-
 	return NULL;
 }
 
@@ -268,11 +301,11 @@ static const char *extchars = "<>=-";
 
 /* end an expression with a delimiter */
 char *
-end_expr(Plisthead *plisthead, char *str)
+end_expr(Plisthead *plisthead, const char *str)
 {
 	Pkglist	*pkglist;
 	char	*expr, *p;
-	
+
 	/* enough space to add a delimiter */
 	XMALLOC(expr, (strlen(str) + 2) * sizeof(char));
 	XSTRCPY(expr, str);
@@ -289,13 +322,13 @@ end_expr(Plisthead *plisthead, char *str)
 			}
 		}
 	}
-	
+
 	/* simple package/dependency  case, parse string backwards
 	 *  until we find a delimiter: foo>=1.0
 	 */
 	if ((p = match_dep_ext(expr, extchars)) != NULL) {
 		*p++ = DELIMITER; /* terminate expr */
-		*p = '\0';		
+		*p = '\0';
 	}
 
 	return expr;
