@@ -1,7 +1,7 @@
-/* $Id: summary.c,v 1.17 2009/05/27 20:42:07 imil Exp $ */
+/* $Id: summary.c,v 1.27 2010/01/03 14:07:29 imil Exp $ */
 
 /*
- * Copyright (c) 2009 The NetBSD Foundation, Inc.
+ * Copyright (c) 2009, 2010 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
  * This code is derived from software contributed to The NetBSD Foundation
@@ -89,39 +89,49 @@ int			colcount = 0;
 /* force pkg_summary reload */
 int			force_fetch = 0;
 
-#ifndef SUMEXT
-#define SUMEXT "bz2"
-#endif
+static const char *const sumexts[] = { "bz2", "gz", NULL };
 
 /* remote summary fetch */
 static char **
 fetch_summary(char *cur_repo)
 {
 	/* from pkg_install/files/admin/audit.c */
-	Dlfile	*file;
+	Dlfile	*file = NULL;
 	char	*decompressed_input;
 	size_t	decompressed_len;
 	time_t	sum_mtime;
+	int		i;
 	char	**out, buf[BUFSIZ];
 
-	if (!force_fetch && !force_update)
-		sum_mtime = pkg_sum_mtime(cur_repo);
-	else
-		sum_mtime = 0;
+	for (i = 0; sumexts[i] != NULL; i++) { /* try all extensions */
+		if (!force_fetch && !force_update)
+			sum_mtime = pkg_sum_mtime(cur_repo);
+		else
+			sum_mtime = 0; /* 0 sumtime == force reload */
 
-	snprintf(buf, BUFSIZ, "%s/%s.%s", cur_repo, PKG_SUMMARY, SUMEXT);
+		snprintf(buf, BUFSIZ, "%s/%s.%s", cur_repo, PKG_SUMMARY, sumexts[i]);
+
+		if ((file = download_file(buf, &sum_mtime)) != NULL)
+			break; /* pkg_summary found and not up-to-date */
+
+		if (sum_mtime < 0) /* pkg_summary found, but up-to-date */
+			return NULL;
+	}
+
+	if (file == NULL) {
+		fprintf(stderr, MSG_COULDNT_FETCH, buf);
 
-	if ((file = download_file(buf, &sum_mtime)) == NULL)
 		return NULL;
+	}
 
 	snprintf(buf, BUFSIZ, UPDATE_REPO_MTIME, (long long)sum_mtime, cur_repo);
 	pkgindb_doquery(buf, NULL, NULL);
 
 	if (decompress_buffer(file->buf, file->size, &decompressed_input,
 			&decompressed_len)) {
-		
+
 		out = splitstr(decompressed_input, "\n");
-		
+
 		XFREE(file->buf);
 		XFREE(file);
 
@@ -141,12 +151,13 @@ progress(char c)
 	const char	*alnum = ALNUM;
 	int 		i, alnumlen = strlen(alnum);
 	float		percent = 0;
-	
+
 	for (i = 0; i < alnumlen; i++)
 		if (c == alnum[i])
 			percent = ((float)(i + 1)/ (float)alnumlen) * 100;
-	
-	printf(MSG_UPDATING_DB, percent);
+
+	printf(MSG_UPDATING_DB_PCT, (int)percent);
+	fflush(stdout);
 }
 
 /* check if the field is PKGNAME */
@@ -165,11 +176,11 @@ static char *
 field_record(const char *field, char *line)
 {
 	char *pfield;
-	
+
 	if (strncmp(field, line, strlen(field)) == 0) {
 		pfield = strchr(line, '=');
 		trimcr(pfield++);
-		
+
 		return pfield;
 	}
 
@@ -180,7 +191,7 @@ static void
 freecols()
 {
 	int i;
-	
+
 	for (i = 0; i < cols.num; i++)
 		XFREE(cols.name[i]);
 
@@ -206,7 +217,7 @@ int
 colnames(void *unused, int argc, char **argv, char **colname)
 {
 	int			i = 0;
-	
+
 	colcount++;
 
 	cols.num = colcount;
@@ -230,7 +241,7 @@ prepare_insert(int pkgid, struct Summary sum, char *cur_repo)
 		query_size = (query_size + strlen(cur_repo)) * sizeof(char);
 
 	XMALLOC(commit_query, query_size);
-	
+
 	snprintf(commit_query, BUFSIZ, "INSERT INTO %s ( PKG_ID,", sum.tbl_name);
 
 	/* insert fields */
@@ -243,7 +254,7 @@ prepare_insert(int pkgid, struct Summary sum, char *cur_repo)
 		XSNPRINTF(commit_query, query_size, "%s\"REPOSITORY\")", commit_query);
 	else
 		commit_query[strlen(commit_query) - 1] = ')';
-	
+
 	XSNPRINTF(commit_query, query_size, "%s VALUES ( %d, ",
 		commit_query, pkgid);
 
@@ -271,11 +282,11 @@ static void
 child_table(int pkgid, const char *table, char *val)
 {
 	char buf[BUFSIZ];
-	
+
 	snprintf(buf, BUFSIZ,
 		"INSERT INTO %s (PKG_ID,%s_PKGNAME) VALUES (%d,\"%s\");",
 		table, table, pkgid, val);
-	
+
 	/* append query to commit_list */
 	commit_idx++;
 	XREALLOC(commit_list, (commit_idx + 1) * sizeof(char *));
@@ -298,7 +309,7 @@ update_col(struct Summary sum, int pkgid, char *line)
 	/* PROVIDES */
 	if ((val = field_record("PROVIDES", line)) != NULL)
 		child_table(pkgid, sum.provides, val);
-	
+
 	for (i = 0; i < cols.num; i++) {
 		snprintf(buf, BUFSIZ, "%s=", cols.name[i]);
 
@@ -307,14 +318,14 @@ update_col(struct Summary sum, int pkgid, char *line)
 		/* XXX: handle that later */
 		if (strncmp(cols.name[i], "DESCRIPTION", 11) == 0)
 			continue;
-		
+
 		if (val != NULL && strncmp(buf, line, strlen(buf)) == 0) {
 			/* nasty little hack to prevent double quotes */
 			if (strchr(line, '"') != NULL)
 				for (p = line; *p != '\0'; p++)
 					if (*p == '"')
 						*p = '`';
-				
+
 
 			XMALLOC(insert, sizeof(Insertlist));
 			XSTRDUP(insert->field, cols.name[i]);
@@ -338,13 +349,11 @@ insert_summary(struct Summary sum, char **summary, char *cur_repo)
 	const char	*alnum = ALNUM;
 	Insertlist	*insert;
 
-	printf( "\e[?25l" ); /* hide the cursor */
-
 	if (summary == NULL) {
 		pkgindb_close();
 		errx(EXIT_FAILURE, "could not read summary");
 	}
-	
+
 	snprintf(query, BUFSIZ, "PRAGMA table_info(%s);", sum.tbl_name);
 
 	/* record columns names to cols */
@@ -355,7 +364,10 @@ insert_summary(struct Summary sum, char **summary, char *cur_repo)
 	XMALLOC(commit_list, sizeof(char *));
 	/* begin transaction */
 	XSTRDUP(commit_list[0], "BEGIN;");
-	
+
+	printf(MSG_UPDATING_DB);
+	fflush(stdout);
+
 	psum = summary;
 	/* main pkg_summary analysis loop */
 	while (*psum != NULL) {
@@ -406,7 +418,7 @@ insert_summary(struct Summary sum, char **summary, char *cur_repo)
 
 		/* reset max query size */
 		query_size = BUFSIZ;
-		
+
 	} /* while *psum != NULL */
 
 	commit_idx++;
@@ -419,7 +431,7 @@ insert_summary(struct Summary sum, char **summary, char *cur_repo)
 		pkgindb_doquery(commit_list[i], NULL, NULL);
 
 	progress(alnum[strlen(alnum) - 1]); /* XXX: nasty. */
-	
+
 	free_list(commit_list);
 	commit_idx = 0;
 
@@ -427,7 +439,7 @@ insert_summary(struct Summary sum, char **summary, char *cur_repo)
 	if (sum.type == LOCAL_SUMMARY)
 		pkgid = 1;
 
-	printf( "\n\e[?25h" ); /* show the cursor again */
+	printf("\n");
 }
 
 static void
@@ -438,8 +450,8 @@ delete_remote_tbl(struct Summary sum, char *repo)
 
 	/* number of elements in sum */
 	nelms = (sizeof(sum) - sizeof(int)) / sizeof(char *) - 1;
-	/* 
-	 * delete repository related tables 
+	/*
+	 * delete repository related tables
 	 * loop through sumsw structure to record table name
 	 * and call associated SQL query
 	 */
@@ -451,7 +463,7 @@ delete_remote_tbl(struct Summary sum, char *repo)
 			continue;
 
 		snprintf(buf, BUFSIZ, DELETE_REMOTE, ptbl, ptbl, repo, ptbl);
-		pkgindb_doquery(buf, NULL, NULL);		
+		pkgindb_doquery(buf, NULL, NULL);
 	}
 
 	snprintf(buf, BUFSIZ,
@@ -496,7 +508,7 @@ update_db(int which, char **pkgkeep)
 	Plisthead	*keeplisthead, *nokeeplisthead, *plisthead;
 	Pkglist		*pkglist;
 	char		**summary = NULL, **prepos, buf[BUFSIZ];
-	
+
 	for (i = 0; i < 2; i++) {
 
 		switch (sumsw[i].type) {
@@ -528,7 +540,6 @@ update_db(int which, char **pkgkeep)
 					pkgindb_doquery(buf, NULL, NULL);
 				}
 				free_pkglist(keeplisthead);
-				XFREE(keeplisthead);
 
 				/*
 				 * packages are installed "manually" by pkgin_install()
@@ -541,7 +552,6 @@ update_db(int which, char **pkgkeep)
 						mark_as_automatic_installed(pkglist->pkgname, 1);
 
 					free_pkglist(nokeeplisthead);
-					XFREE(nokeeplisthead);
 				}
 			} else { /* empty keep list */
 				/*
@@ -557,7 +567,6 @@ update_db(int which, char **pkgkeep)
 						}
 
 					free_pkglist(plisthead);
-					XFREE(plisthead);
 				}
 			}
 
